0010***********************************************************************
0020* PROGRAM-ID.        EXL100RA
0030* PROGRAM NAME.      Interface to Excel & Workbook
0040* PROGRAMMER NAME.   C. Scott
0050* ANALYST NAME.      B. Engele
0060*
0070*    *** SPECIAL PROCEDURES ***
0080*        Source should only exist in the PC DBA-EXL1 library, the
0090*        object should be copied to PC SYSTEM FUSR.
0100*
0110*
0120*    *** STATUS LOG ***
0130*  ---------------------------------------------------------------------
0140*  DD/MMM/CCYY | CD | PROGRAMMER | DESCRIPTION OF CHANGES
0150*  ------------+----+------------+--------------------------------------
0160*  15/Jul/2005 | AA | C. Scott   | New External SubRoutine
0170*  ---------------------------------------------------------------------
0180*
0190*
0200* @@@ START SSSAPP ODM SECTIONS
0210*
0220*    *** PROGRAM NARRATIVE ***
0230*    Interface to create a pointer to MS Excel application and various
0240*    functions for Workbooks (files including .xls, .csv, etc.).
0250*    This interface can be used to:
0260*      1. Open an existing workbook and return it's attributes.
0270*      2. Create a new workbook and set attributes for it.
0280*      3. Open an existing workbook and save each sheet as a separate
0290*         comma separated value (csv) file.
0300*      4. Close an open workbook and end the Excel session.
0310*
0320*  THIS PROGRAM IS CALLED FROM     Various including RPC
0330*
0340*  THIS PROGRAM EXITS TO PROGRAM   Calling routine
0350*      ON    CONDITION
0360*  OR TO                           SSAAPPIV
0370*      ON    CONDITION
0380*
0390*    *** CONTROL STATEMENTS ***
0400*
0410*  --------------------------------------------------------------------
0420*  | COLUMNS |            |        |     |     |
0430*  | FR | TO | FIELD NAME | SOURCE | LNG | TYP | COMMENTS
0440*  -----+----+------------+--------+-----+-----+-----------------------
0450*  |    |    |            |        |     |     |
0460*  --------------------------------------------------------------------
0470*
0480*    *** FORMS ***
0490*
0500*  --------------------------------------------------------------------
0510*  |   FORM   | RPT # |  DESCRIPTION
0520*  -----------+-------+------------------------------------------------
0530*  | SSSAPPF0 |  9999 |
0540*  --------------------------------------------------------------------
0550*
0560* @@@ END ODM SECTIONS
0570***********************************************************************
0580/*Struct Off
0590Define Data
0600  Parameter Using EXL-OBJ1
0610  Parameter Using EXL100P1
0620  Local Using EXL100L1
0630End-Define
0640/*Struct On
0650*
0660********************************
0670Define Subroutine Excel-WorkBook
0680********************************
0690Reset #EXL-Error-No #EXL-Error-Msg
0700If #EXL-Application = NULL-HANDLE
0710** *  Instanciate Excel Object and set for non-interactive mode  * **
0720** *  unless the flag to show Excel and its' alerts is set.      * **
0730  Create Object #EXL-Application Of Class "EXCEL.APPLICATION"
0740  If Not #EXL100-SHOW-EXCEL
0750     #EXL-Application.Visible := False
0760     #EXL-Application.DisplayAlerts := False
0770  Else
0780     #EXL-Application.Visible := True
0790     #EXL-Application.DisplayAlerts := True  /* Show confirmation popups
0800  End-If
0810** *  Setting these options did not seem to make any difference  * **
0820** *  even for a vary large sheet with many forulas.             * **
0830*   Send "Calculation" To #EXL-Application With -4135 /* xlCalculationManual
0840*      Giving #EXL-Error-No
0850*   Send "CalculateBeforeSave" To #EXL-Application With False
0860*      Giving #EXL-Error-No
0870*
0880** *  Dispatch Workbooks Interface  * **
0890  #EXL-WorkBooks := #EXL-Application.WORKBOOKS
0900End-If
0910*
0920Create Object #oFSO Of Class "SCRIPTING.FILESYSTEMOBJECT"
0930*
0940#WorkBook-Command := *Translate(#EXL-Command, Upper)
0950Decide on First Value of #WorkBook-Command
0960  Value 'OPEN'
0970     Perform Open-Excel-WorkBook
0980  Value 'CREATE'
0990     Perform Create-Excel-WorkBook
1000  Value 'CONVERTCSV'
1010     Perform Open-Excel-WorkBook
1020     If #EXL-Error-No = 0
1030        Perform Convert-Excel-WorkBook
1040        #EXL100-SAVE-CHANGES := False
1050        Perform Close-Excel-WorkBook
1060     End-If
1070  Value 'SAVE'
1080     Perform Save-Excel-WorkBook
1090  Value 'CLOSE'
1100*      Send "Calculation" To #EXL-Application With -4105 /* xlCalculationAutomatic
1110*         Giving #EXL-Error-No
1120*      Send "CalculateBeforeSave" To #EXL-Application With False
1130*         Giving #EXL-Error-No
1140     Perform Close-Excel-WorkBook
1150  None Value
1160     #EXL-Error-No := 1
1170     Compress 'EXL100 - Invalid command:' #EXL-Command 'used, please correct'
1180        Into #EXL-Error-Msg
1190     Escape Routine
1200End-Decide
1210#oFSO := NULL-HANDLE
1220*
1230*************************************
1240Define Subroutine Open-Excel-WorkBook
1250*************************************
1260** *  Open an existing Excel WorkBook and return sheet info  * **
1270  Compress #EXL100-Path '\' #EXL100-FileName Into #PathFileName
1280     Leaving No SPace
1290  Send "FileExists" To #oFSO With #PathFileName Return #FileExists
1300  If #FileExists
1310     Send "Workbooks.Open" To #EXL-WorkBooks With #PathFileName
1320     Return #EXL-WorkBook Giving #EXL-Error-No
1330     If #EXL-Error-No ne 0
1340        Compress 'EXL100 - Error from open for file' #PathFileName
1350           Into #EXL-Error-Msg
1360        Escape Routine
1370     End-if
1380     Perform Get-WorkBook-Sheets
1390  Else
1400     #EXL-Error-No := 2
1410     #EXL-Error-Msg := 'EXL100 - File specified does not exist.'
1420     Escape Routine
1430  End-If
1440End-Subroutine /* Open-Excel-WorkBook
1450*************************************
1460*
1470***************************************
1480Define Subroutine Create-Excel-WorkBook
1490***************************************
1500** *  First ensure that the workbook being created doesn't already  * **
1510** *  exist, if it does return an error??  rename old one??         * **
1520  Compress #EXL100-Path '\' #EXL100-FileName Into #PathFileName
1530     Leaving No SPace
1540  Send "FileExists" To #oFSO With #PathFileName Return #FileExists
1550  If #FileExists
1560** *  Rename the old file by appending Date/Time  * **
1570     Move Edited *TIMX (EM='_'DDLLLYYYY'-'HHIISS'.') To #Date-Time
1580     #NewFileName := #PathFileName
1590     Examine Direction Backward #NewFileName For '.'
1600        Replace With #Date-Time
1610     Send "MoveFile" To #oFSO With #PathFileName #NewFileName
1620        Giving #EXL-Error-No
1630  End-IF
1640** *  Perform Add method on Workbooks interface to add a new workbook
1650  #EXL-WorkBook := #EXL-WorkBooks.ADD
1660  If #EXL100-NumSheets = 0
1670     Perform Get-WorkBook-Sheets
1680  Else  /* Set the number and names of the worksheets in new workbook
1690     Perform Set-WorkBook-Sheets
1700  End-If
1710  Send "SaveAs" To #EXL-WorkBook With #PathFileName /* 11X
1720     Giving #EXL-Error-No
1730  If #EXL-Error-No NE 0
1740     #First-Error := #EXL-Error-No
1750     Compress 'EXL100 - Error' #EXL-Error-No 'returned from trying to create new'
1760        'Excel file' #PathFileName Into #EXL-Error-Msg
1770     #EXL100-Save-Changes := False
1780     Perform Close-Excel-WorkBook
1790     #EXL-Error-No := #First-Error
1800     Escape Routine
1810  End-If
1820End-Subroutine /* Create-Excel-WorkBook
1830***************************************
1840*
1850****************************************
1860Define Subroutine Convert-Excel-WorkBook
1870****************************************
1880** *  Used to convert the workbook into CSV file(s) using the  * **
1890** *  sheet name(s) as the prefix for the file name(s).        * **
1900** *  The full path and file name(s) are returned in the       * **
1910** *  #EXL100-SheetNames dynamic array.                        * **
1920  For #Loop = 1 to #EXL100-NumSheets
1930     Compress #EXL100-Path '\' #EXL100-SheetNames(#Loop) '.csv'
1940        Into #EXL100-SheetNames(#Loop) Leaving No Space
1950     Send "Item" To #EXL-Sheets With #LOOP
1960        Return #EXL-Sheet Giving #EXL-Error-No
1970     Send "Activate" To #EXL-Sheet Giving #EXL-Error-No
1980     #CSV-FileName := #EXL100-SheetNames(#Loop)
1990** *  Check to see if the csv file already exists, if it does  * **
2000** *  just rename it and create the new one.                   * **
2010     Send "FileExists" To #oFSO With #CSV-FileName Return #FileExists
2020     If #FileExists
2030** *  Rename the old file by appending Date/Time  * **
2040        Move Edited *TIMX (EM='_'DDLLLYYYY'-'HHIISS'.') To #Date-Time
2050        #NewFileName := #CSV-FileName
2060        Examine Direction Backward #NewFileName For '.'
2070           Replace With #Date-Time
2080        Send "MoveFile" To #oFSO With #CSV-FileName #NewFileName
2090           Giving #EXL-Error-No
2100     End-If
2110     #Excel-Version := #EXL-Application.Version
2120     If #Excel-Version LE '8.0e' /* Excel97 or before?
2130** *  The SaveAs method requires the "optional" parms to be  * **
2140** *  specified since it is an overloaded method.            * **
2150        Send "SaveAs" To #EXL-Sheet With #CSV-FileName #xlCSV 7X
2160           Giving #EXL-Error-No
2170     Else
2180        Send "SaveAs" To #EXL-Sheet With #CSV-FileName #xlCSV 4X
2190           Return #CSV-WorkBook Giving #EXL-Error-No
2200     End-If
2210     If #EXL-Error-No NE 0
2220        Compress 'EXL100 - Error:' #EXL-Error-No 'from saving CSV file:'
2230           #EXL100-SheetNames(#Loop) Into #EXL-Error-Msg
2240        Escape Bottom
2250     End-if
2260  End-For
2270  #CSV-WorkBook := NULL-HANDLE
2280End-Subroutine /* Convert-Excel-WorkBook
2290****************************************
2300*
2310*************************************
2320Define Subroutine Save-Excel-WorkBook
2330*************************************
2340  If #EXL-WorkBook NE NULL-HANDLE
2350     #Changes-Saved := #EXL-WorkBook.Saved
2360** *  Only do the save if there have been changes since the last save * **
2370     If Not #Changes-Saved
2380        Send "Save" To #EXL-WorkBook Giving #EXL-Error-No
2390        If #EXL-Error-No NE 0
2400           Compress 'EXL100 - Error' #EXL-Error-No 'returned, changes not saved!'
2410              Into #EXL-Error-Msg
2420        End-If
2430     End-If
2440  End-If
2450End-Subroutine /* Save-Excel-WorkBook
2460*************************************
2470*
2480**************************************
2490Define Subroutine Close-Excel-WorkBook
2500**************************************
2510  Send "Close" To #EXL-WorkBook With #EXL100-Save-Changes 2X
2520  Include EXL-Done
2530End-Subroutine /* Close-Excel-WorkBook
2540****************************************
2550*
2560*************************************************************
2570*************************************************************
2580** *  Other common code to be used by this routine only  * **
2590**************************************************************
2600*
2610*************************************
2620Define Subroutine Get-WorkBook-Sheets
2630*************************************
2640** *  Get a pointer to the collection of sheets on the workbook  * **
2650  #EXL-Sheets := #EXL-WorkBook.WORKSHEETS
2660** *  Get the number of sheets in the current workbook
2670  #EXL100-NumSheets := #EXL-Sheets.COUNT
2680  If #EXL100-NumSheets GT 0
2690     Expand Array #EXL100-SheetNames To (*:#EXL100-NumSheets)
2700     For #Loop = 1 to #EXL100-NumSheets
2710        Send "SHEETS.ITEM" To #EXL-Sheets With #LOOP
2720           Return #EXL-Sheet Giving #EXL-Error-No
2730        If #EXL-Error-No ne 0
2740           Compress 'EXL100 - Error:' #EXL-Error-No 'from getting sheet:' #Loop
2750              Into #EXL-Error-Msg
2760           Escape Routine
2770        End-if
2780        #EXL100-SheetNames(#Loop) := #EXL-Sheet.NAME
2790     End-For
2800  End-If
2810End-Subroutine /* Get-WorkBook-Sheets
2820*************************************
2830*
2840*************************************
2850Define Subroutine Set-WorkBook-Sheets
2860*************************************
2870  If #EXL100-NumSheets NE *OCC(#EXL100-SheetNames)
2880     Resize Array #EXL100-SheetNames To (*:#EXL100-NumSheets)
2890  End-If
2900** *  Get a pointer to the collection of sheets on the workbook  * **
2910  #EXL-Sheets := #EXL-WorkBook.WORKSHEETS
2920** *  Get the number of sheets in the current workbook
2930  #Temp-NumSheets := #EXL-Sheets.COUNT
2940  If #Temp-NumSheets NE #EXL100-NumSheets
2950     If #Temp-NumSheets LT #EXL100-NumSheets
2960** *  Add to end so get handle to last sheet  * **
2970        Send "Item" To #EXL-Sheets With #Temp-NumSheets
2980           Return #EXL-Sheet Giving #EXL-Error-No
2990        #Temp-NumSheets := #EXL100-NumSheets - #Temp-NumSheets
3000        Send "Add" To #EXL-Sheets With 1X #EXL-Sheet #Temp-NumSheets
3010           Return #EXL-Sheet Giving #EXL-Error-No
3020     Else
3030** *  Remove the end sheets that are not needed  * **
3040        #Temp-NumSheets := #EXL100-NumSheets + 1
3050        Send "Item" To #EXL-Sheets With #Temp-NumSheets
3060           Return #EXL-Sheet Giving #EXL-Error-No
3070        Repeat Until #EXL-Sheet = NULL-HANDLE or #EXL-Error-No NE 0
3080           Send "Delete" TO #EXL-Sheet
3090           Send "Item" To #EXL-Sheets With #Temp-NumSheets
3100              Return #EXL-Sheet Giving #EXL-Error-No
3110        End-Repeat
3120     End-If
3130  End-If
3140  For #Loop = 1 to #EXL100-NumSheets
3150     Send "Item" To #EXL-Sheets With #LOOP
3160        Return #EXL-Sheet Giving #EXL-Error-No
3170     If #EXL-Error-No ne 0
3180        Compress 'EXL100 - Error:' #EXL-Error-No 'from getting sheet:' #Loop
3190           Into #EXL-Error-Msg
3200        Escape Routine
3210     End-If
3220     If #EXL100-SheetNames(#Loop) NE ' '
3230** *  For some reason can't directly assign from a Dynamic X-array  * **
3240        #Temp-SheetName := #EXL100-SheetNames(#Loop)
3250        #EXL-Sheet.NAME := #Temp-SheetName
3260     Else
3270        #EXL100-SheetNames(#Loop) := #EXL-Sheet.NAME
3280     End-If
3290  End-For
3300End-Subroutine /* Set-WorkBook-Sheets
3310*************************************
3320*
3330**************************************************************
3340** *  End of common code to be used by this routine only  * **
3350**************************************************************
3360**************************************************************
3370*
3380On Error
3390  #EXL-Error-No := *Error-Nr
3400  Compress 'Error' #EXL-Error-No 'occurred in' *Program 'on line'
3410     *Error-Line 'and could not be handled by the routine.'
3420     Into #EXL-Error-Msg
3430  Escape Module
3440End-Error
3450*
3460End-Subroutine /* Excel-WorkBook
3470********************************
3480*
3490End
