0010***********************************************************************
0020* PROGRAM-ID.        EXL102RA
0030* PROGRAM NAME.      Interface to Excel worksheet cells
0040* PROGRAMMER NAME.   C. Scott
0050* ANALYST NAME.      B. Engele
0060*
0070*    *** SPECIAL PROCEDURES ***
0080*        Source should only exist in the PC DBA-EXL1 library, the
0090*        object should be copied to PC SYSTEM FUSR.
0100*
0110*
0120*    *** STATUS LOG ***
0130*  ---------------------------------------------------------------------
0140*  DD/MMM/CCYY | CD | PROGRAMMER | DESCRIPTION OF CHANGES
0150*  ------------+----+------------+--------------------------------------
0160*  03/Aug/2006 | AF | C. Scott   | Add AutoFilter & FreezePane functions
0170*                                | (Loreen's suggestion!)
0180*  ---------------------------------------------------------------------
0190*  10/Jul/2006 | AE | C. Scott   | Fix outlined DATA-GROUP function by
0200*                                | using the Rows or Columns property.
0210*                                | Add Merge Range function
0220*  ---------------------------------------------------------------------
0230*  05/Jul/2006 | AD |Barry Engele| In the UPDATE portion of the
0240*                                | CELL-COMMAND DECIDE stmt the ESCAPE
0250*                                | BOTTOM was chged to ESCAPE-ROUTINE
0260*  ---------------------------------------------------------------------
0270*  27/Apr/2006 | AC | C. Scott   | Add Outlined Data Group function
0280*  ---------------------------------------------------------------------
0290*  20/Mar/2006 | AB | C. Scott   | Add Named cell function
0300*  ---------------------------------------------------------------------
0310*  01/Sep/2005 | AA | C. Scott   | New External SubRoutine
0320*  ---------------------------------------------------------------------
0330*
0340*
0350/*STRUCT OFF
0360* @@@ START SSSAPP ODM SECTIONS
0370*
0380*    *** PROGRAM NARRATIVE ***
0390*    Interface to use the MS Excel application and various functions
0400*    for workSheet cells.
0410*    This interface can be used to:
0420*      1. Select cell(s) within the active worksheet.
0430*      2. Add data to cell(s) of the active worksheet.
0440*      3. Add formula(s) to cell(s) of the active worksheet.
0450*      4. Define the format for selected cell(s)
0460*      4. Return the address of a cell given numeric location.
0470*      5. Allow a cell to be given a NAME other than cell location
0480*      6. Allow rows or columns to be specified and create an outlined
0490*           data group to allow showing and hiding levels of detail.
0500*      7. Merge a range of cells together so they act like a single cell.
0510*      8. Enable AutoFilter for column(s)
0520*      9. Activate the FreezePane option for the worksheet
0530*
0540*  THIS PROGRAM IS CALLED FROM     Various including RPC
0550*
0560*  THIS PROGRAM EXITS TO PROGRAM   Calling routine
0570*      ON    CONDITION
0580*  OR TO                           SSAAPPIV
0590*      ON    CONDITION
0600*
0610*    *** CONTROL STATEMENTS ***
0620*
0630*  --------------------------------------------------------------------
0640*  | COLUMNS |            |        |     |     |
0650*  | FR | TO | FIELD NAME | SOURCE | LNG | TYP | COMMENTS
0660*  -----+----+------------+--------+-----+-----+-----------------------
0670*  |    |    |            |        |     |     |
0680*  --------------------------------------------------------------------
0690*
0700*    *** FORMS ***
0710*
0720*  --------------------------------------------------------------------
0730*  |   FORM   | RPT # |  DESCRIPTION
0740*  -----------+-------+------------------------------------------------
0750*  | SSSAPPF0 |  9999 |
0760*  --------------------------------------------------------------------
0770*
0780* @@@ END ODM SECTIONS
0790***********************************************************************
0800Define Data
0810  Parameter Using EXL-OBJ1
0820  Parameter Using EXL102P1
0830  Local Using EXL102L1
0840End-Define
0850*
0860*****************************
0870Define Subroutine Excel-Cells
0880*****************************
0890Reset #EXL-Error-No #EXL-Error-Msg
0900If #EXL-Application = NULL-HANDLE or #EXL-WorkBook = NULL-HANDLE
0910   or #EXL-Sheets = NULL-HANDLE or #EXL-Sheet = NULL-HANDLE
0920** *  It's an error at this point, there needs to be an open workbook  * **
0930** *  before any of the worksheet functions can be performed.          * **
0940  #EXL-Error-No := 1021
0950  #EXL-Error-Msg := 'EXL102 - There must be an open workbook before' -
0960     'using this routine!'
0970  Escape Routine
0980End-If
0990*
1000Send "Activate" to #EXL-Sheet Giving #EXL-Error-No
1010If #EXL-Error-No = 0
1020  #EXL-Sheet-Cells := #EXL-Application.Cells
1030Else
1040  #EXL-Error-Msg := 'EXL102 - Error activating current sheet'
1050  Escape Routine
1060End-If
1070*
1080#Cell-Command := *Translate(#EXL-Command, Upper)
1090Decide on First Value of #Cell-Command
1100  Value 'SELECT'
1110     Perform Select-Cell-Range
1120  Value 'UPDATE'
1130** *  Figure out how many optional data parameters passed, there can  * **
1140** *  only be one at a time since a single range of cells is selected * **
1150     If *OCC(#EXL102-Cell-Data-Text,1) NE 0 And *OCC(#EXL102-Cell-Data-Numeric,1) NE 0
1160        #EXL-Error-No := 1023
1170        #EXL-Error-Msg := 'EXL102 - Must pass ONE type of cell data' -
1180           ' at a time for update!'
1190        Escape Routine
1200     End-If
1210** *  If a range has not been selected already do it now  * **
1220     If #EXL-Cell-Range = NULL-HANDLE
1230        Perform Select-Cell-Range
1240     End-If
1250     If *OCC(#EXL102-Cell-Data-Numeric,1) NE 0 /* Specified
1260        #EXL-Cell-Range.Value := #EXL102-Cell-Data-Numeric(*,*)
1270     Else
1280        #EXL-Cell-Range.Value := #EXL102-Cell-Data-Text(*,*)
1290     End-If
1300     If #EXL102-Range-Number-Format Specified
1310        #EXL-Cell-Range.NumberFormat := #EXL102-Range-Number-Format
1320     End-IF
1330  Value 'FORMULA'
1340** *  Formulas can be relative when applied to a range of cells  * **
1350** *  unless the $ qualifier is used in the cell locations.      * **
1360     If #EXL102-Range-Formula Specified
1370           and #EXL102-Range-Formula = Mask('='*)
1380** *  If a range has not been selected already do it now  * **
1390        If #EXL-Cell-Range = NULL-HANDLE
1400           Perform Select-Cell-Range
1410        End-If
1420        #EXL-Cell-Range.Formula := #EXL102-Range-Formula
1430     Else
1440        #EXL-Error-No := 1024
1450        #EXL-Error-Msg := 'EXL102 - A valid Excel formula must be' -
1460           ' supplied in the #EXL102-Range-Formula parameter!'
1470        Escape Routine
1480     End-If
1490     If #EXL102-Range-Number-Format Specified
1500        #EXL-Cell-Range.NumberFormat := #EXL102-Range-Number-Format
1510     End-IF
1520  Value 'NUMBER-FORMAT'
1530** *  If a range has not been selected already do it now  * **
1540     If #EXL-Cell-Range = NULL-HANDLE
1550        Perform Select-Cell-Range
1560     End-If
1570     If #EXL102-Range-Number-Format Specified /* Set the numeric mask
1580           And #EXL102-Range-Number-Format NE ' '
1590        #EXL-Cell-Range.NumberFormat := #EXL102-Range-Number-Format
1600     Else
1610        #EXL-Error-No := 1025
1620        #EXL-Error-Msg := 'EXL102 - Valid Excel formatting' -
1630           ' information must be supplied in appropriate parameter!'
1640        Escape Routine
1650     End-If
1660  Value 'NAME-RANGE'
1670     If #EXL102-Range-Name Specified  /* Req'd for the name
1680** *  Need the sheet name to build the RefersTo part for name  * **
1690        #Refers-To := #EXL-Sheet.Name
1700        If #EXL102-Start-Cell-Col-Id = ' '
1710           #Cell-Col := #EXL102-Range-Start-Col
1720           Perform Translate-Column-Address
1730        Else
1740           #Cell-Col-Id := #EXL102-Start-Cell-Col-Id
1750        End-If
1760        #Cell-Row := #EXL102-Range-Start-Row
1770        Compress '=''' #Refers-To '''!$' #Cell-Col-Id '$' #Cell-Row
1780           Into #Refers-To Leaving No Space
1790        If #EXL102-Range-End-Row NE 0  /* Named range rather than cell
1800           If #EXL102-End-Cell-Col-Id = ' '
1810              #Cell-Col := #EXL102-Range-End-Col
1820              Perform Translate-Column-Address
1830           Else
1840              #Cell-Col-Id := #EXL102-End-Cell-Col-Id
1850           End-If
1860           #Cell-Row := #EXL102-Range-End-Row
1870           Compress #Refers-To ':' #Cell-Col-Id '$' #Cell-Row
1880              Into #Refers-To Leaving No Space
1890        End-If
1900** *  Get a handle to the workbook NAMES object  * **
1910        #WorkBook-Names := #EXL-WorkBook.Names
1920        Send "Add" To #WorkBook-Names With #EXL102-Range-Name #Refers-To
1930           RETURN #Range-Name Giving #EXL-Error-No
1940        If #EXL-Error-No NE 0
1950           Compress 'EXL102 - Error from add of range name'
1960              #EXL102-Range-Name 'for' #Refers-To Into #EXL-Error-Msg
1970        End-If
1980     Else
1990        #EXL-Error-No := 1026
2000        #EXL-Error-Msg := 'EXL102 - The name must be supplied in the' -
2010           ' #EXL102-Range-Name parameter!'
2020        Escape Routine
2030     End-If
2040  Value 'ADDRESS'  /* Return the alpha cell ids from the indexes
2050     If (#EXL102-Range-Start-Col + #EXL102-Range-End-Col) GT 0
2060        If #EXL102-Range-Start-Col GT 0
2070           #Cell-Col := #EXL102-Range-Start-Col
2080           Perform Translate-Column-Address
2090           #EXL102-Start-Cell-Col-Id := #Cell-Col-Id
2100        End-IF
2110        If #EXL102-Range-End-Col GT 0
2120           #Cell-Col := #EXL102-Range-End-Col
2130           Perform Translate-Column-Address
2140           #EXL102-End-Cell-Col-Id := #Cell-Col-Id
2150        End-If
2160     Else
2170        #EXL-Error-No := 1027
2180        #EXL-Error-Msg := 'EXL102 - The column number(s) must be specified!'
2190        Escape Routine
2200     End-If
2210  Value 'COLUMN-NUMBER'  /* Translate the column ID(s) to the number
2220     If #EXL102-Start-Cell-Col-Id NE ' ' Or #EXL102-End-Cell-Col-Id NE ' '
2230        If #EXL102-Start-Cell-Col-Id NE ' '
2240           #Cell-Col-Id := #EXL102-Start-Cell-Col-Id
2250           Perform Translate-Col-To-Number
2260           #EXL102-Range-Start-Col := #Cell-Col
2270        End-If
2280        If #EXL102-End-Cell-Col-Id NE ' '
2290           #Cell-Col-Id := #EXL102-End-Cell-Col-Id
2300           Perform Translate-Col-To-Number
2310           #EXL102-Range-End-Col := #Cell-Col
2320        End-If
2330     Else
2340        #EXL-Error-No := 1028
2350        #EXL-Error-Msg := 'EXL102 - The column id(s) must be specified!'
2360        Escape Routine
2370     End-If
2380  Value 'DATA-GROUP'
2390     Decide For First Condition
2400        When (#Exl102-Range-Start-Row + #Exl102-Range-End-Row) GT 1
2410           Compress #Exl102-Range-Start-Row ':' #Exl102-Range-End-Row
2420              Into #Range-Id Leaving No Space
2430           Send "Rows" To #EXL-Sheet-Cells With #Range-Id
2440              Return #EXL-Cell-Range Giving #EXL-Error-No
2450           If #EXL-Error-No NE 0
2460              #EXL-Error-Msg := 'EXL102 - Error returned from selecting'
2470              - ' specified rows.'
2480              Escape Routine
2490           End-If
2500        When #EXL102-Start-Cell-Col-Id NE ' ' And #EXL102-End-Cell-Col-Id NE ' '
2510           Compress #EXL102-Start-Cell-Col-Id ':' #EXL102-End-Cell-Col-Id
2520              Into #Range-Id Leaving No Space
2530           Send "Columns" To #EXL-Sheet-Cells With #Range-Id
2540              Return #EXL-Cell-Range Giving #EXL-Error-No
2550           If #EXL-Error-No NE 0
2560              #EXL-Error-Msg := 'EXL102 - Error returned from selecting'
2570              -  'specified columns.'
2580              Escape Routine
2590           End-If
2600        When None
2610           #EXL-Error-No := 1029
2620           #EXL-Error-Msg := 'EXL102 - The start and end must be' -
2630              ' specified to create a Data Group.'
2640           Escape Routine
2650     End-Decide
2660     Send "Group" TO #EXL-Cell-Range Giving #EXL-Error-No
2670     If #EXL-Error-No NE 0
2680        #EXL-Error-Msg := 'EXL102 - Error returned from creating the Data Group.'
2690        Escape Routine
2700     End-If
2710** *  By default the group still shows all detail, allow it to be  * **
2720** *  hidden if the NO-DETAIL command option is used.              * **
2730     If #EXL102-Command-Option Specified
2740        #EXL102-Command-Option := *Translate(#EXL102-Command-Option, Upper)
2750        If #EXL102-Command-Option = 'NO-DETAIL'
2760           If (#Exl102-Range-Start-Row + #Exl102-Range-End-Row) GT 1
2770              #Range-Id := #Exl102-Range-End-Row + 1
2780              Send "Rows" To #EXL-Sheet-Cells With #Range-Id
2790                 Return #EXL-Cell-Range Giving #EXL-Error-No
2800              If #EXL-Error-No NE 0
2810                 #EXL-Error-Msg := 'EXL102 - Error returned from selecting'
2820                 - ' specified rows.'
2830                 Escape Routine
2840              End-If
2850           Else
2860              #Cell-Col-Id := #EXL102-End-Cell-Col-Id
2870              Perform Translate-Col-To-Number
2880              Add 1 To #Cell-Col
2890              Perform Translate-Column-Address
2900              #Range-Id := #Cell-Col-Id
2910              Send "Columns" To #EXL-Sheet-Cells With #Range-Id
2920                 Return #EXL-Cell-Range Giving #EXL-Error-No
2930              If #EXL-Error-No NE 0
2940                 #EXL-Error-Msg := 'EXL102 - Error returned from selecting'
2950                 -  'specified columns.'
2960                 Escape Routine
2970              End-If
2980           End-If
2990           #EXL-Cell-Range.ShowDetail := False
3000        End-If
3010     End-If
3020  Value 'MERGE-RANGE'
3030** *  If a range has not been selected already do it now  * **
3040     If #EXL-Cell-Range = NULL-HANDLE
3050        If (#Exl102-Range-Start-Row + #Exl102-Range-End-Row +
3060                    #Exl102-Range-Start-Col + #Exl102-Range-End-Col) GT 4
3070           Perform Select-Cell-Range
3080        Else
3090           #EXL-Error-No := 1024
3100           #EXL-Error-Msg := 'EXL102 - The range start and end must' -
3110              ' be specified to merge cells.'
3120           Escape Routine
3130        End-IF
3140     End-If
3150     Send "Merge" To #EXL-Cell-Range Giving #EXL-Error-No
3160     If #EXL-Error-No NE 0
3170        #EXL-Error-Msg := 'EXL102 - Error returned from merging the specified cells.'
3180        Escape Routine
3190     End-If
3200  Value 'AUTOFILTER'
3210** *  If a range has not been selected already do it now  * **
3220     If #EXL-Cell-Range = NULL-HANDLE
3230        If #Exl102-Range-Start-Row GT 0 And #Exl102-Range-Start-Col GT 0
3240           If NOT (#Exl102-Range-End-Row GT 0 And #Exl102-Range-End-Col GT 0)
3250              #Exl102-Range-End-Row := #Exl102-Range-Start-Row
3260              #Exl102-Range-End-Col := #Exl102-Range-Start-Col
3270           End-If
3280           Perform Select-Cell-Range
3290        Else
3300           #EXL-Error-No := 1025
3310           #EXL-Error-Msg := 'EXL102 - The start row and column must' -
3320              ' be specified for the AutoFilter function.'
3330           Escape Routine
3340        End-IF
3350     End-If
3360     Send "AutoFilter" To #EXL-Cell-Range Giving #EXL-Error-No
3370     If #EXL-Error-No NE 0
3380        #EXL-Error-Msg := 'EXL102 - Error returned from AutoFilter method'
3390        Escape Routine
3400     End-If
3410  Value 'FREEZEPANE'
3420     If #Exl102-Range-Start-Row GT 0 And #Exl102-Range-Start-Col GT 0
3430        #Cell-Row := #Exl102-Range-Start-Row
3440        #Cell-Col := #Exl102-Range-Start-Col
3450        Perform Select-Cell
3460     Else
3470        #EXL-Error-No := 1025
3480        #EXL-Error-Msg := 'EXL102 - The start row and column must' -
3490           ' be specified for the FreezePane function.'
3500        Escape Routine
3510     End-If
3520** *  Need to activate the cell in the worksheet before the    * **
3530** *  freeze pane function will use it as the freeze position  * **
3540     Send "Activate" To #Selected-Cell Giving #EXL-Error-No
3550     If #EXL-Error-No NE 0
3560        #EXL-Error-Msg := 'EXL102 - Error activating the required cell'
3570           - ' for the FreezePane function.'
3580        Escape Routine
3590     End-If
3600** *  The FreezePane property is only available through the  * **
3610** *  active window object.                                  * **
3620     Send "ActiveWindow" To #EXL-Application 
3630        Return #oWindow Giving #EXL-Error-No
3640     If #EXL-Error-No NE 0
3650        #EXL-Error-Msg := 'EXL102 - Error accessing the Active Window'
3660           - ' for the FreezePane function.'
3670        Escape Routine
3680     End-If
3690     #oWindow.FreezePanes := True
3700  None Value
3710     #EXL-Error-No := 1029
3720     Compress 'EXL102 - Invalid command:' #EXL-Command 'used.'
3730        Into #EXL-Error-Msg
3740End-Decide
3750Reset #EXL-Command
3760*
3770*************************************************************
3780*************************************************************
3790** *  Other common code to be used by this routine only  * **
3800*************************************************************
3810*
3820*******************************
3830  Define Subroutine Select-Cell
3840*******************************
3850     Send "Range.Cells" TO #EXL-Sheet-Cells With #Cell-Row #Cell-Col
3860        Return #Selected-Cell Giving #EXL-Error-No
3870     If #EXL-Error-No NE 0
3880        Compress 'EXL102 - Error from selection of cell at row' #Cell-Row
3890           'column' #Cell-Col Into #EXL-Error-Msg
3900        Escape Module
3910     End-If
3920  End-Subroutine /* Select-Cell
3930*******************************
3940*
3950*************************************
3960  Define Subroutine Select-Cell-Range
3970*************************************
3980     If (#EXL102-Range-Start-Row + #EXL102-Range-Start-Col +
3990           #EXL102-Range-End-Row + #EXL102-Range-End-Row) LT 4
4000        #EXL-Error-No := 1022
4010        #EXL-Error-Msg := 'All of the start and end values for the' -
4020           ' range must be provided'
4030        Escape Module
4040     End-If
4050     #Cell-Row := #EXL102-Range-Start-Row
4060     #Cell-Col := #EXL102-Range-Start-Col
4070     Perform Select-Cell
4080     #Range-Start-Cell := #Selected-Cell
4090*
4100     #Cell-Row := #EXL102-Range-End-Row
4110     #Cell-Col := #EXL102-Range-End-Col
4120     Perform Select-Cell
4130     #Range-End-Cell := #Selected-Cell
4140*
4150     Send "Range" To #EXL-Sheet-Cells With #Range-Start-Cell
4160        #Range-End-Cell Return #EXL-Cell-Range Giving #EXL-Error-No
4170     If #EXL-Error-No NE 0
4180        #EXL-Error-Msg := 'EXL102 - Error from selection of cell range specified'
4190        Escape Module
4200     End-If
4210  End-Subroutine /* Select-Cell-Range
4220*************************************
4230*
4240********************************************
4250  Define Subroutine Translate-Column-Address
4260********************************************
4270** *  Translate the column number into an excel column id  * **
4280     Divide 26 Into #Cell-Col Remainder #Col-Remainder
4290     If #Col-Remainder = 0
4300        #Col-Remainder := 26
4310        If #Cell-Col GT 0
4320           Subtract 1 from #Cell-Col
4330        End-If
4340     End-If
4350     Compress #Column-Translation(#Cell-Col) #Column-Translation(#Col-Remainder)
4360        Into #Cell-Col-Id Leaving No Space
4370  End-Subroutine /* Translate-Column-Address
4380********************************************
4390*
4400*******************************************
4410  Define Subroutine Translate-Col-To-Number
4420*******************************************
4430** *  Translate the excel column id into the numeric column  * **
4440     Examine #Column-Translation(*) For #Cell-First-Col-Id
4450        Giving Index In #Cell-Col
4460     If #Cell-Second-Col-Id NE ' '  /* handle double char column id's
4470        Examine #Column-Translation(*) For #Cell-Second-Col-Id
4480           Giving Index In #Col-Remainder
4490        #Cell-Col := #Cell-Col * 26 + #Col-Remainder
4500     End-If
4510  End-Subroutine /* Translate-Col-To-Number
4520*******************************************
4530*
4540**************************************************************
4550** *  End of common code to be used by this routine only  * **
4560**************************************************************
4570**************************************************************
4580*
4590On Error
4600  #EXL-Error-No := *Error-Nr
4610  Compress 'Error' #EXL-Error-No 'occurred in' *Program 'on line'
4620     *Error-Line 'and could not be handled by the routine.'
4630     Into #EXL-Error-Msg
4640  Escape Module
4650End-Error
4660*
4670End-Subroutine /* Excel-Cells
4680*****************************
4690*
4700End
