0010***********************************************************************
0020* PROGRAM-ID.        EXL103RA
0030* PROGRAM NAME.      Interface to format Excel worksheet cells
0040* PROGRAMMER NAME.   C. Scott
0050* ANALYST NAME.      B. Engele
0060*
0070*    *** SPECIAL PROCEDURES ***
0080*        Source should only exist in the PC DBA-EXL1 library, the
0090*        object should be copied to PC SYSTEM FUSR.
0100*
0110*
0120*    *** STATUS LOG ***
0130*  ---------------------------------------------------------------------
0140*  DD/MMM/CCYY | CD | PROGRAMMER | DESCRIPTION OF CHANGES
0150*  ------------+----+------------+--------------------------------------
0160*  31/Jul/2006 | AA | C. Scott   | New External SubRoutine
0170*  ---------------------------------------------------------------------
0180*
0190*
0200* @@@ START SSSAPP ODM SECTIONS
0210*
0220*    *** PROGRAM NARRATIVE ***
0230*    Interface to use the MS Excel application and various functions
0240*    to format workSheet cells.
0250*    This interface can be used to set properties on cells & fonts
0260*
0270** *  cell/Range Properties  * **
0280* Locked := (T/F)  /* Used when sheet is protected??
0290*
0300** *  Cell Interior properties  * **
0310* ColorIndex := XL-COLOR
0320* Pattern := XL-CELLS
0330* PatternColorIndex := XL-COLOR
0340*
0350** *  Cell Border properties  * **
0360* ColorIndex := XL-COLOR
0370* LineStyle := XL-BORDR
0380* Weight := XL-BORDR
0390*
0400** *  Font Properties  * **
0410* Background := XL-FONT - #XL-FONT-BACKGROUND
0420* Bold := XL-FONT - #XL-TRUE-FALSE
0430* ColorIndex := XL-COLOR
0440* Italic := XL-FONT - #XL-TRUE-FALSE
0450* OutlineFont := XL-FONT - #XL-TRUE-FALSE
0460* Shadow := XL-FONT - #XL-TRUE-FALSE
0470* Size := Integer - point value
0480* Strikethrough := XL-FONT - #XL-TRUE-FALSE
0490* Subscript := XL-FONT - #XL-TRUE-FALSE
0500* Superscript := XL-FONT - #XL-TRUE-FALSE
0510* Underline := XL-FONT - #XL-FONT-UNDERLINE-STYLE
0520*
0530*
0540*  THIS PROGRAM IS CALLED FROM     Various including RPC
0550*
0560*  THIS PROGRAM EXITS TO PROGRAM   Calling routine
0570*      ON    CONDITION
0580*  OR TO                           SSAAPPIV
0590*      ON    CONDITION
0600*
0610*    *** CONTROL STATEMENTS ***
0620*
0630*  --------------------------------------------------------------------
0640*  | COLUMNS |            |        |     |     |
0650*  | FR | TO | FIELD NAME | SOURCE | LNG | TYP | COMMENTS
0660*  -----+----+------------+--------+-----+-----+-----------------------
0670*  |    |    |            |        |     |     |
0680*  --------------------------------------------------------------------
0690*
0700*    *** FORMS ***
0710*
0720*  --------------------------------------------------------------------
0730*  |   FORM   | RPT # |  DESCRIPTION
0740*  -----------+-------+------------------------------------------------
0750*  | SSSAPPF0 |  9999 |
0760*  --------------------------------------------------------------------
0770*
0780* @@@ END ODM SECTIONS
0790***********************************************************************
0800Define Data
0810  Parameter Using EXL-OBJ1
0820  Parameter Using EXL103P1
0830  Local Using EXL103L1
0840End-Define
0850*
0860***********************************
0870Define Subroutine Excel-Cell-Format
0880***********************************
0890Reset #EXL-Error-No #EXL-Error-Msg
0900If #EXL-Application = NULL-HANDLE or #EXL-WorkBook = NULL-HANDLE
0910   or #EXL-Sheets = NULL-HANDLE or #EXL-Sheet = NULL-HANDLE
0920** *  It's an error at this point, there needs to be an open workbook  * **
0930** *  before any of the worksheet functions can be performed.          * **
0940  #EXL-Error-No := 1031
0950  #EXL-Error-Msg := 'EXL103 - There must be an open workbook before' -
0960     'using this routine!'
0970  Escape Routine
0980End-If
0990*
1000Send "Activate" to #EXL-Sheet Giving #EXL-Error-No
1010If #EXL-Error-No = 0
1020  #EXL-Sheet-Cells := #EXL-Application.Cells
1030Else
1040  #EXL-Error-Msg := 'EXL103 - Error activating current sheet'
1050  Escape Routine
1060End-If
1070*
1080If #EXL103-Formatting-Options = ' '
1090  #EXL-Error-No := 1032
1100  #EXL-Error-Msg := 'EXL103 - Formatting options are required!'
1110  Escape Module
1120End-If
1130*
1140If #EXL-Cell-Range = NULL-HANDLE
1150  Perform Select-Cell-Range
1160End-If
1170*
1180#Format-Command := *Translate(#EXL-Command, Upper)
1190Decide on First Value of #Format-Command
1200  Value 'CELL-FORMAT'
1210     Perform Separate-Property-Values
1220     For #Loop = 1 To #Total-Properties
1230        If #Value(#Loop) IS (I4)
1240           #Integer := Val(#Value(#Loop))
1250        Else
1260           Perform Invalid-Value-For-Property
1270        End-If
1280        #Property := *Translate(#Properties(#Loop), Upper)
1290        Decide On First Value of #Property
1300           Value 'CELLCOLOR', 'PATTERN', 'PATTERNCOLOR'
1310              If #oInterior = NULL-HANDLE
1320                 Send "Interior" To #EXL-Cell-Range Return #oInterior
1330                    Giving #EXL-Error-No
1340                 If #EXL-Error-No NE 0
1350                    #EXL-Error-Msg := 'EXL103 - Error accessing cell interior object!'
1360                    Escape Module
1370                 End-IF
1380              End-If
1390              Perform Set-Cell-Interior
1400           Value 'HORIZONTALALIGNMENT'
1410              #EXL-Cell-Range.HorizontalAlignment := #Integer
1420           Value 'VERTICALALIGNMENT'
1430              #EXL-Cell-Range.VerticalAlignment := #Integer
1440           Value 'BORDERLOCATION'  /* Select the border to work with
1450              If #Integer = 5 Thru 12
1460                 Send "Borders" To #EXL-Cell-Range With #Integer 
1470                    Return #oBorders Giving #EXL-Error-No
1480                 If #EXL-Error-No NE 0
1490                    #EXL-Error-Msg := 
1500                       'EXL103 - Error accessing cell border object!'
1510                    Escape Module
1520                 End-IF
1530              Else
1540                 Perform Invalid-Value-For-Property
1550              End-If
1560           Value 'BORDERCOLOR', 'BORDERSTYLE', 'BORDERWEIGHT'
1570              Perform Set-Border-Properties
1580           None Value
1590              #EXL-Error-No := 1033
1600              Compress 'EXL103 - Invalid formatting property:'
1610                 #Properties(#Loop) 'used for cell formatting'
1620                 Into #EXL-Error-Msg
1630              Escape Module
1640        End-Decide
1650     End-For
1660  Value 'FONT-FORMAT'
1670     Send "Font" To #EXL-Cell-Range Return #oFont
1680        Giving #EXL-Error-No
1690     If #EXL-Error-No NE 0
1700        #EXL-Error-Msg := 'EXL103 - Error accessing cell font object!'
1710        Escape Module
1720     End-IF
1730     Perform Separate-Property-Values
1740     For #Loop = 1 To #Total-Properties
1750        #Property := *Translate(#Properties(#Loop), Upper)
1760        If #Value(#Loop) IS (I4)
1770           #Integer := Val(#Value(#Loop))
1780        Else
1790           If #Value(#Loop) = 'True' Or = 'False'
1800              Move Edited #Value(#Loop) To #Logical (EM=False/True)
1810           Else
1820              Perform Invalid-Value-For-Property
1830           End-If
1840        End-If
1850        Decide on First Value of #Property
1860           Value 'BACKGROUND'
1870              #oFont.Background := #Integer
1880           Value 'COLOR'
1890              #oFont.ColorIndex := #Integer
1900           Value 'SIZE'
1910              #oFont.Size := #Integer
1920           Value 'UNDERLINE'
1930              #oFont.Underline := #Integer
1940           Value 'BOLD'
1950              #oFont.Bold := #Logical
1960           Value 'ITALIC'
1970              #oFont.Italic := #Logical
1980           Value 'OUTLINE'
1990              #oFont.OutlineFont := #Logical
2000           Value 'SHADOW'
2010              #oFont.Shadow := #Logical
2020           Value 'STRIKETHROUGH'
2030              #oFont.Strikethrough := #Logical
2040           Value 'SUBSCRIPT'
2050              #oFont.Subscript := #Logical
2060           Value 'SUPERSCRIPT'
2070              #oFont.Superscript := #Logical
2080           None Value
2090              #EXL-Error-No := 1033
2100              Compress 'EXL103 - Invalid formatting property:'
2110                 #Properties(#Loop) 'used for font formatting'
2120                 Into #EXL-Error-Msg
2130              Escape Module
2140        End-Decide
2150     End-For
2160  None Value
2170     #EXL-Error-No := 1034
2180     #EXL-Error-Msg := 'EXL103 - Undefined formatting command used.'
2190     Escape Module
2200End-Decide
2210*
2220*************************************************************
2230*************************************************************
2240** *  Other common code to be used by this routine only  * **
2250*************************************************************
2260*
2270*******************************
2280  Define Subroutine Select-Cell
2290*******************************
2300     Send "Range.Cells" TO #EXL-Sheet-Cells With #Cell-Row #Cell-Col
2310        Return #Selected-Cell Giving #EXL-Error-No
2320     If #EXL-Error-No NE 0
2330        Compress 'EXL103 - Error from selection of cell at row' #Cell-Row
2340           'column' #Cell-Col Into #EXL-Error-Msg
2350        Escape Module
2360     End-If
2370  End-Subroutine /* Select-Cell
2380*******************************
2390*
2400*************************************
2410  Define Subroutine Select-Cell-Range
2420*************************************
2430     If (#EXL103-Range-Start-Row + #EXL103-Range-Start-Col +
2440           #EXL103-Range-End-Row + #EXL103-Range-End-Row) LT 4
2450        #EXL-Error-No := 1034
2460        #EXL-Error-Msg := 'EXL103 - All of the start and end values for'
2470           - ' the range must be provided'
2480        Escape Module
2490     End-If
2500     #Cell-Row := #EXL103-Range-Start-Row
2510     #Cell-Col := #EXL103-Range-Start-Col
2520     Perform Select-Cell
2530     #Range-Start-Cell := #Selected-Cell
2540*
2550     #Cell-Row := #EXL103-Range-End-Row
2560     #Cell-Col := #EXL103-Range-End-Col
2570     Perform Select-Cell
2580     #Range-End-Cell := #Selected-Cell
2590*
2600     Send "Range" To #EXL-Sheet-Cells With #Range-Start-Cell
2610        #Range-End-Cell Return #EXL-Cell-Range Giving #EXL-Error-No
2620     If #EXL-Error-No NE 0
2630        #EXL-Error-Msg := 'EXL103 - Error from selection of cell range specified'
2640        Escape Module
2650     End-If
2660  End-Subroutine /* Select-Cell-Range
2670*************************************
2680*
2690********************************************
2700  Define Subroutine Separate-Property-Values
2710********************************************
2720     Examine #EXL103-FORMATTING-OPTIONS For ',' 
2730        Giving Number in #Total-Properties
2740     Add 1 To #Total-Properties
2750     Expand Array #Properties To (*:#Total-Properties)
2760     Expand Array #Value To (*:#Total-Properties)
2770     If #Total-Properties GT 1
2780        Expand Array #Property-And-Value To (*:#Total-Properties)
2790        Separate #EXL103-FORMATTING-OPTIONS Into #Property-And-Value(*)
2800           With Delimiter ','
2810        For #Loop = 1 To #Total-Properties
2820           Separate #Property-And-Value(#Loop) Into #Properties(#Loop)
2830              #Value(#Loop) WITH DELIMITER '='
2840        End-For
2850     Else
2860        Separate #EXL103-FORMATTING-OPTIONS Into #Properties(1)
2870           #Value(1) WITH DELIMITER '='
2880     End-If
2890     Reduce Array #Property-And-Value To 0
2900  End-Subroutine /* Separate-Property-Values
2910********************************************
2920*
2930*************************************
2940  Define Subroutine Set-Cell-Interior
2950*************************************
2960     Decide On First Value of #Property
2970        Value 'CELLCOLOR'
2980           #oInterior.ColorIndex := #Integer
2990        Value 'PATTERN'
3000           #oInterior.Pattern := #Integer
3010        Value 'PATTERNCOLOR'
3020           #oInterior.PatternColorIndex := #Integer
3030        None Value  /* Already checked for valid value
3040           Ignore
3050     End-Decide
3060  End-Subroutine /* Set-Cell-Interior
3070*************************************
3080*
3090***************************************
3100  Define Subroutine Set-Border-Properties
3110***************************************
3120     Decide On First Value of #Property
3130        Value 'BORDERCOLOR'
3140           #oBorders.ColorIndex := #Integer
3150        Value 'BORDERSTYLE'
3160           #oBorders.LineStyle := #Integer
3170        Value 'BORDERWEIGHT'
3180           #oBorders.Weight := #Integer
3190        None Value  /* Already checked for valid values
3200           Ignore
3210     End-Decide
3220  End-Subroutine /* Set-Border-Properties
3230***************************************
3240*
3250**********************************************
3260  Define Subroutine Invalid-Value-For-Property
3270**********************************************
3280     #EXL-Error-No := 1035
3290     Compress 'EXL103 - Invalid value of' #Value(#Loop) 'for' 
3300        #Properties(#Loop) 'property' Into #EXL-Error-Msg
3310     Escape Module
3320  End-Subroutine /* Invalid-Value-For-Property
3330**********************************************
3340*
3350********
3360On Error
3370  #EXL-Error-No := *Error-Nr
3380  #EXL-Error-Msg := 'EXL103 - Unhandled error occurred.'
3390  Escape Module
3400End-Error
3410*********
3420*
3430End-Subroutine /* Excel-Cell-Format
3440***********************************
3450End
3460*
3470* SEND "BORDERS" TO #RANGE WITH #XLEDGELEFT RETURN #BORDERS
3480* #BORDERS.WEIGHT := #XLMEDIUM
3490* Worksheets("Sheet1").Range("A1:A5").HorizontalAlignment = xlLeft
3500* Worksheets("Sheet1").Rows(2).VerticalAlignment = xlVAlignCenter
3510* Worksheets("Sheet1").Range("A1").Interior.ColorIndex = 8  ' Cyan
3520*
3530
3540
